package service

// Code generated by gowrap. DO NOT EDIT.
// template: ../../../assets/templates/middleware/access_log
// gowrap: http://github.com/hexdigest/gowrap

//go:generate gowrap gen -p git.perx.ru/perxis/perxis-go/pkg/invitations -i Invitations -t ../../../assets/templates/middleware/access_log -o logging_middleware.go -l ""

import (
	"context"
	"fmt"
	"time"

	"git.perx.ru/perxis/perxis-go/pkg/auth"
	"git.perx.ru/perxis/perxis-go/pkg/invitations"
	"git.perx.ru/perxis/perxis-go/pkg/options"
	"go.uber.org/zap"
	"go.uber.org/zap/zapcore"
)

// loggingMiddleware implements invitations.Invitations that is instrumented with logging
type loggingMiddleware struct {
	logger *zap.Logger
	next   invitations.Invitations
}

// LoggingMiddleware instruments an implementation of the invitations.Invitations with simple logging
func LoggingMiddleware(logger *zap.Logger) Middleware {
	return func(next invitations.Invitations) invitations.Invitations {
		return &loggingMiddleware{
			next:   next,
			logger: logger,
		}
	}
}

func (m *loggingMiddleware) Accept(ctx context.Context, invitationId string, userId string) (err error) {
	begin := time.Now()
	var fields []zapcore.Field
	for k, v := range map[string]interface{}{
		"ctx":          ctx,
		"invitationId": invitationId,
		"userId":       userId} {
		if k == "ctx" {
			fields = append(fields, zap.String("principal", fmt.Sprint(auth.GetPrincipal(ctx))))
			continue
		}
		fields = append(fields, zap.Reflect(k, v))
	}

	m.logger.Debug("Accept.Request", fields...)

	err = m.next.Accept(ctx, invitationId, userId)

	fields = []zapcore.Field{
		zap.Duration("time", time.Since(begin)),
		zap.Error(err),
	}

	for k, v := range map[string]interface{}{
		"err": err} {
		if k == "err" {
			continue
		}
		fields = append(fields, zap.Reflect(k, v))
	}

	m.logger.Debug("Accept.Response", fields...)

	return err
}

func (m *loggingMiddleware) Create(ctx context.Context, invitation *invitations.Invitation) (created *invitations.Invitation, err error) {
	begin := time.Now()
	var fields []zapcore.Field
	for k, v := range map[string]interface{}{
		"ctx":        ctx,
		"invitation": invitation} {
		if k == "ctx" {
			fields = append(fields, zap.String("principal", fmt.Sprint(auth.GetPrincipal(ctx))))
			continue
		}
		fields = append(fields, zap.Reflect(k, v))
	}

	m.logger.Debug("Create.Request", fields...)

	created, err = m.next.Create(ctx, invitation)

	fields = []zapcore.Field{
		zap.Duration("time", time.Since(begin)),
		zap.Error(err),
	}

	for k, v := range map[string]interface{}{
		"created": created,
		"err":     err} {
		if k == "err" {
			continue
		}
		fields = append(fields, zap.Reflect(k, v))
	}

	m.logger.Debug("Create.Response", fields...)

	return created, err
}

func (m *loggingMiddleware) Delete(ctx context.Context, invitationId string) (err error) {
	begin := time.Now()
	var fields []zapcore.Field
	for k, v := range map[string]interface{}{
		"ctx":          ctx,
		"invitationId": invitationId} {
		if k == "ctx" {
			fields = append(fields, zap.String("principal", fmt.Sprint(auth.GetPrincipal(ctx))))
			continue
		}
		fields = append(fields, zap.Reflect(k, v))
	}

	m.logger.Debug("Delete.Request", fields...)

	err = m.next.Delete(ctx, invitationId)

	fields = []zapcore.Field{
		zap.Duration("time", time.Since(begin)),
		zap.Error(err),
	}

	for k, v := range map[string]interface{}{
		"err": err} {
		if k == "err" {
			continue
		}
		fields = append(fields, zap.Reflect(k, v))
	}

	m.logger.Debug("Delete.Response", fields...)

	return err
}

func (m *loggingMiddleware) Find(ctx context.Context, filter *invitations.Filter, opts *options.FindOptions) (invitations []*invitations.Invitation, total int, err error) {
	begin := time.Now()
	var fields []zapcore.Field
	for k, v := range map[string]interface{}{
		"ctx":    ctx,
		"filter": filter,
		"opts":   opts} {
		if k == "ctx" {
			fields = append(fields, zap.String("principal", fmt.Sprint(auth.GetPrincipal(ctx))))
			continue
		}
		fields = append(fields, zap.Reflect(k, v))
	}

	m.logger.Debug("Find.Request", fields...)

	invitations, total, err = m.next.Find(ctx, filter, opts)

	fields = []zapcore.Field{
		zap.Duration("time", time.Since(begin)),
		zap.Error(err),
	}

	for k, v := range map[string]interface{}{
		"invitations": invitations,
		"total":       total,
		"err":         err} {
		if k == "err" {
			continue
		}
		fields = append(fields, zap.Reflect(k, v))
	}

	m.logger.Debug("Find.Response", fields...)

	return invitations, total, err
}

func (m *loggingMiddleware) Get(ctx context.Context, invitationId string) (invitation *invitations.Invitation, err error) {
	begin := time.Now()
	var fields []zapcore.Field
	for k, v := range map[string]interface{}{
		"ctx":          ctx,
		"invitationId": invitationId} {
		if k == "ctx" {
			fields = append(fields, zap.String("principal", fmt.Sprint(auth.GetPrincipal(ctx))))
			continue
		}
		fields = append(fields, zap.Reflect(k, v))
	}

	m.logger.Debug("Get.Request", fields...)

	invitation, err = m.next.Get(ctx, invitationId)

	fields = []zapcore.Field{
		zap.Duration("time", time.Since(begin)),
		zap.Error(err),
	}

	for k, v := range map[string]interface{}{
		"invitation": invitation,
		"err":        err} {
		if k == "err" {
			continue
		}
		fields = append(fields, zap.Reflect(k, v))
	}

	m.logger.Debug("Get.Response", fields...)

	return invitation, err
}
